<?php

class ModuleCustomer extends AppModule
{
  function __construct($System)
  {
    parent::__construct($System);
    $this->Name = 'Customer';
    $this->Version = '1.0';
    $this->Creator = 'Chronos';
    $this->License = 'GNU/GPL';
    $this->Description = 'Customer management';
    $this->Dependencies = array('User');
  }

  function DoStart()
  {
    $this->System->FormManager->RegisterClass('Member', array(
      'Title' => 'Zákazníci',
      'Table' => 'Member',
      'DefaultSortColumn' => 'Name',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Jméno', 'Default' => ''),
        'ResponsibleUser' => array('Type' => 'TUser', 'Caption' => 'Zodpovědný uživatel', 'Default' => ''),
        'Subject' => array('Type' => 'TSubject', 'Caption' => 'Subjekt', 'Default' => ''),
        'FamilyMemberCount' => array('Type' => 'String', 'Caption' => 'Osob v domácnosti', 'Default' => '0', 'Suffix' => 'osob'),
        'MembershipDate' => array('Type' => 'Date', 'Caption' => 'Datum členství', 'Default' => ''),
        'BillingPeriod' => array('Type' => 'TFinanceBillingPeriod', 'Caption' => 'Fakturační období aktuální', 'Default' => ''),
        'BillingPeriodNext' => array('Type' => 'TFinanceBillingPeriod', 'Caption' => 'Fakturační období příští', 'Default' => ''),
        'BillingPeriodLastDate' => array('Type' => 'Date', 'Caption' => 'Datum poslední fakturace', 'Default' => ''),
        'Blocked' => array('Type' => 'Boolean', 'Caption' => 'Blokování', 'Default' => '0'),
        'PayDay' => array('Type' => 'Integer', 'Caption' => 'Den placení', 'Default' => '1', 'Suffix' => 'den'),
        'Devices' => array('Type' => 'TDeviceList', 'Caption' => 'Registrovaná zařízení', 'Default' => ''),
        'UserRel' => array('Type' => 'TUserCustomerRelListCustomer', 'Caption' => 'Přiřazení uživatelé', 'Default' => ''),
        'ServiceRel' => array('Type' => 'TServiceCustomerRelListCustomer', 'Caption' => 'Placené služby', 'Default' => ''),
        'Consumption' => array('Type' => 'TCustomerStockSerialNumber', 'Caption' => 'Spotřeba zařízení', 'Default' => ''),
      ),
      'ItemActions' => array(
        array('Caption' => 'Klientská sekce', 'URL' => '/finance/platby/?i=#RowId'),
      ),
    ));
    $this->System->FormManager->RegisterClass('MemberPayment', array(
      'Title' => 'Placení zákazníků',
      'Table' => 'MemberPayment',
      'Items' => array(
        'Member' => array('Type' => 'TMember', 'Caption' => 'Zákazník', 'Default' => '', 'ReadOnly' => true),
        'MonthlyTotal' => array('Type' => 'Integer', 'Caption' => 'Celkem měsíčně', 'Default' => '0', 'Suffix' => 'Kč', 'ReadOnly' => true),
        'MonthlyInternet' => array('Type' => 'Integer', 'Caption' => 'Internet měsíčně', 'Default' => '', 'Suffix' => 'Kč', 'ReadOnly' => true),
        'MonthlyConsumption' => array('Type' => 'Integer', 'Caption' => 'Spotřeba měsíčně', 'Default' => '', 'Suffix' => 'Kč', 'ReadOnly' => true),
        'MonthlyPlus' => array('Type' => 'Integer', 'Caption' => 'Měsíčně plus', 'Default' => '', 'Suffix' => 'Kč', 'ReadOnly' => true),
        'Cash' => array('Type' => 'Integer', 'Caption' => 'Kredit', 'Default' => '', 'Suffix' => 'Kč', 'ReadOnly' => true),
      ),
      'Actions' => array(
        array('Caption' => 'Přepočítat', 'URL' => '/finance/sprava/?Operation=Recalculate'),
      ),
    ));
    $this->System->FormManager->RegisterFormType('TMember', array(
      'Type' => 'Reference',
      'Table' => 'Member',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterClass('Service', array(
      'Title' => 'Služby',
      'Table' => 'Service',
      'DefaultSortColumn' => 'Name',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Jméno', 'Default' => ''),
        'Category' => array('Type' => 'TServiceCategory', 'Caption' => 'Skupina', 'Default' => '', 'Null' => true),
        'Price' => array('Type' => 'Integer', 'Caption' => 'Cena', 'Default' => '0', 'Suffix' => 'Kč'),
        'VAT' => array('Type' => 'TFinanceVATType', 'Caption' => 'Sazba DPH', 'Default' => '0', 'Suffix' => ''),
        'CustomerCount' => array('Type' => 'Integer', 'Caption' => 'Počet zákazníků', 'Default' => '', 'ReadOnly' => true,
          'SQL' => '(SELECT COUNT(*) FROM `ServiceCustomerRel` LEFT JOIN `Member` ON `Member`.`Id`=`ServiceCustomerRel`.`Customer` WHERE (`ServiceCustomerRel`.`Service`=#Id) AND (`Member`.`Blocked`=0))'),
        'Public' => array('Type' => 'Boolean', 'Caption' => 'Veřejné', 'Default' => ''),
        'InternetSpeedMin' => array('Type' => 'Integer', 'Caption' => 'Min. rychlost internetu', 'Default' => '0', 'Suffix' => 'kbit/s'),
        'InternetSpeedMax' => array('Type' => 'Integer', 'Caption' => 'Max. rychlost internetu', 'Default' => '0', 'Suffix' => 'kbit/s'),
        'UploadAsymmetry' => array('Type' => 'Integer', 'Caption' => 'Asymetrie odesílání', 'Default' => '1'),
        'Memory' => array('Type' => 'Integer', 'Caption' => 'Paměť', 'Default' => '0', 'Suffix' => 'GB'),
        'MemorySwap' => array('Type' => 'Integer', 'Caption' => 'Odkládací oddíl', 'Default' => '0', 'Suffix' => 'GB'),
        'Storage' => array('Type' => 'Integer', 'Caption' => 'Úložiště', 'Default' => '0', 'Suffix' => 'GB'),
        'CPUCount' => array('Type' => 'Integer', 'Caption' => 'Počet jader', 'Default' => '0', 'Suffix' => ''),
        'ChangeAction' => array('Type' => 'TActionEnum', 'Caption' => 'Změna - akce', 'Default' => '', 'Null' => true),
        'ChangeTime' => array('Type' => 'DateTime', 'Caption' => 'Změna - čas', 'Default' => '', 'Null' => true),
        'ChangeReplaceId' => array('Type' => 'TService', 'Caption' => 'Změna - položka', 'Default' => '', 'Null' => true),
        'CustomerRel' => array('Type' => 'TServiceCustomerRelListService', 'Caption' => 'Placení zákazníky', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('ServiceCategory', array(
      'Title' => 'Kategorie služeb',
      'Table' => 'ServiceCategory',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Jméno', 'Default' => ''),
        'Items' => array('Type' => 'TServiceListServiceCategory', 'Caption' => 'Služby', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('ServiceCustomerRel', array(
      'Title' => 'Vztahy služba - zákazník',
      'Table' => 'ServiceCustomerRel',
      'Items' => array(
        'Service' => array('Type' => 'TService', 'Caption' => 'Služba', 'Default' => ''),
        'Customer' => array('Type' => 'TMember', 'Caption' => 'Zákazník', 'Default' => ''),
        'ChangeAction' => array('Type' => 'TActionEnum', 'Caption' => 'Změna - akce', 'Default' => '', 'Null' => true),
        'ChangeTime' => array('Type' => 'DateTime', 'Caption' => 'Změna - čas', 'Default' => '', 'Null' => true),
        'ChangeReplaceId' => array('Type' => 'TServiceCustomerRel', 'Caption' => 'Změna - položka', 'Default' => '', 'Null' => true),
      ),
    ));
    $this->System->FormManager->RegisterFormType('TServiceCategory', array(
      'Type' => 'Reference',
      'Table' => 'ServiceCategory',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TService', array(
      'Type' => 'Reference',
      'Table' => 'Service',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TServiceCustomerRel', array(
      'Type' => 'Reference',
      'Table' => 'ServiceCustomerRel',
      'Id' => 'Id',
      'Name' => 'Id',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TServiceCustomerRelListCustomer', array(
      'Type' => 'ManyToOne',
      'Table' => 'ServiceCustomerRel',
      'Id' => 'Id',
      'Ref' => 'Customer',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TServiceCustomerRelListService', array(
      'Type' => 'ManyToOne',
      'Table' => 'ServiceCustomerRel',
      'Id' => 'Id',
      'Ref' => 'Service',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TServiceListServiceCategory', array(
      'Type' => 'ManyToOne',
      'Table' => 'Service',
      'Id' => 'Id',
      'Ref' => 'Category',
      'Filter' => '1',
    ));

    $this->System->ModuleManager->Modules['IS']->RegisterDashboardItem('Customer',
      array('ModuleCustomer', 'ShowDashboardItem'));
  }

  function ShowDashboardItem()
  {
    $DbResult = $this->Database->select('Member', 'COUNT(*)', '1');
    $DbRow = $DbResult->fetch_row();
    $Output = 'Zákazníků: registrovaných:'.$DbRow['0'];

    $DbResult = $this->Database->select('Member', 'COUNT(*)', '`Blocked`=0 AND `BillingPeriod`<>1');
    $DbRow = $DbResult->fetch_row();
    $Output .= ' platících:'.$DbRow['0'].'<br/>';

    return($Output);
  }

}
